@@ -5,7 +5,7 @@ use Carp qw/croak confess carp/;
 
 BEGIN {
   $Math::Prime::Util::GMP::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::GMP::VERSION = '0.27';
+  $Math::Prime::Util::GMP::VERSION = '0.41';
 }
 
 # parent is cleaner, and in the Perl 5.10.1 / 5.12.0 core, but not earlier.
@@ -19,8 +19,12 @@ our @EXPORT_OK = qw(
                      is_provable_prime_with_cert
                      is_aks_prime
                      is_nminus1_prime
+                     is_nplus1_prime
+                     is_bls75_prime
                      is_ecpp_prime
                      is_pseudoprime
+                     is_euler_pseudoprime
+                     is_euler_plumb_pseudoprime
                      is_strong_pseudoprime
                      is_lucas_pseudoprime
                      is_strong_lucas_pseudoprime
@@ -29,12 +33,21 @@ our @EXPORT_OK = qw(
                      is_perrin_pseudoprime
                      is_frobenius_pseudoprime
                      is_frobenius_underwood_pseudoprime
+                     is_frobenius_khashin_pseudoprime
+                     is_mersenne_prime
+                     is_llr_prime
+                     is_proth_prime
+                     is_miller_prime
                      miller_rabin_random
-                     lucas_sequence
+                     lucas_sequence  lucasu  lucasv
                      primes
                      sieve_primes
+                     sieve_twin_primes
+                     sieve_prime_cluster
+                     sieve_range
                      next_prime
                      prev_prime
+                     surround_primes
                      trial_factor
                      prho_factor
                      pbrent_factor
@@ -45,24 +58,33 @@ our @EXPORT_OK = qw(
                      ecm_factor
                      qs_factor
                      factor
+                     divisors
+                     sigma
+                     chinese
                      moebius
                      prime_count
                      primorial
                      pn_primorial
                      factorial
                      consecutive_integer_lcm
-                     partitions bernfrac stirling
-                     gcd lcm kronecker valuation invmod binomial gcdext
+                     partitions bernfrac bernreal harmfrac harmreal stirling
+                     zeta riemannr
+                     gcd lcm kronecker valuation binomial gcdext
+                     invmod sqrtmod addmod mulmod divmod powmod
                      vecsum vecprod
                      exp_mangoldt
                      liouville
                      totient
                      jordan_totient
                      carmichael_lambda
-                     is_power
+                     sqrtint rootint
+                     is_power is_prime_power
+                     is_primitive_root
                      znorder
                      znprimroot
+                     ramanujan_tau
                      Pi
+                     todigits
                    );
                    # Should add:
                    # nth_prime
@@ -88,8 +110,9 @@ sub _validate_positive_integer {
   if (ref($n) eq 'Math::BigInt' && $n->can("sign")) {
     croak "Parameter '$n' must be a positive integer" unless $n->sign() eq '+';
   } else {
-    croak "Parameter '$n' must be a positive integer"
-          if $n eq '' || $n =~ tr/0123456789//c;
+    my $sn = "$n";
+    croak "Parameter '$sn' must be a positive integer"
+          if $sn eq '' || $sn =~ tr/0123456789//c;
   }
   croak "Parameter '$n' must be >= $min" if defined $min && $n < $min;
   croak "Parameter '$n' must be <= $max" if defined $max && $n > $max;
@@ -97,17 +120,6 @@ sub _validate_positive_integer {
 }
 
 
-sub is_strong_pseudoprime {
-  my($n, @bases) = @_;
-  _validate_positive_integer($n);
-  croak "No bases given to is_strong_pseudoprime" unless @bases;
-  foreach my $base (@bases) {
-    _validate_positive_integer($base);
-    return 0 unless _GMP_miller_rabin("$n", "$base");
-  }
-  1;
-}
-
 sub is_provable_prime {
   my ($n) = @_;
   return 0 if $n < 2;
@@ -128,29 +140,12 @@ sub is_provable_prime_with_cert {
   return ($result, $text);
 }
 
-sub factor {
-  my ($n) = @_;
-  my @factors = ($n < 4) ? ($n)
-                         : sort {$a<=>$b} _GMP_factor($n);
-  return @factors;
-}
-
 sub primes {
-  my $optref = (ref $_[0] eq 'HASH')  ?  shift  :  {};
-  croak "no parameters to primes" unless scalar @_ > 0;
-  croak "too many parameters to primes" unless scalar @_ <= 2;
-  my $low = (@_ == 2)  ?  shift  :  2;
-  my $high = shift;
+  my($low,$high) = (scalar(@_) == 1) ? (2,$_[0]) : ($_[0], $_[1]);
 
   _validate_positive_integer($low);
   _validate_positive_integer($high);
 
-  return [] if $low > $high || $high < 2;
-
-  # Trial if small size.
-  return _GMP_trial_primes($low, $high) if $low < 2_000_000_000;
-
-  # Use the partial sieve with BPSW post-filter.
   [ sieve_primes($low, $high, 0) ];
 }
 
@@ -165,7 +160,7 @@ __END__
 
 =encoding utf8
 
-=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac stirling
+=for stopwords Möbius Deléglise Bézout gcdext vecsum vecprod moebius totient liouville znorder znprimroot bernfrac bernreal harmfrac harmreal stirling zeta riemannr lucasu lucasv OpenPFGW gmpy2 nonresidue chinese tuplets sqrtmod addmod mulmod powmod divmod superset sqrtint rootint todigits
 
 =head1 NAME
 
@@ -174,7 +169,7 @@ Math::Prime::Util::GMP - Utilities related to prime numbers and factoring, using
 
 =head1 VERSION
 
-Version 0.27
+Version 0.41
 
 
 =head1 SYNOPSIS
@@ -300,17 +295,23 @@ This call walks the line between the performance of L</is_prob_prime>
 and the certainty of L</is_provable_prime>.  Those calls may be more
 appropriate in some cases.  What this function does is give most of
 the performance of the former, while adding more certainty.  For finer
-tuning of this tradeoff, especially if performance is critical for
-65- to 200-bit inputs, you may instead use L</is_prob_prime> followed
+tuning of this tradeoff, you may instead use L</is_prob_prime> followed
 by additional probable prime tests such as L</miller_rabin_random>
 and/or L</is_frobenius_underwood_pseudoprime>.
 
-As with L</is_prob_prime>, a BPSW test is first performed.  If this
-indicates "probably prime" then a small number of Miller-Rabin tests
-with random bases are performed.  For numbers under 200 bits, a quick
+As with L</is_prob_prime>, a BPSW test is first performed.  This is
+deterministic for all 64-bit numbers.  Next, if the number is a
+Proth or LLR form, then a proof is constructed.  If the result is
+still "probably prime" and the input is smaller than the
+Sorenson/Webster (2015) deterministic Miller-Rabin limit
+(approximately 82 bits) then the 11 or 12 Miller-Rabin tests are
+performed and the result is confirmed.  For larger inputs that are
+still "probably prime" but under 200 bits, a quick
 BLS75 C<n-1> primality proof is attempted.  This is tuned to give up
 if the result cannot be quickly determined, and results in success
 rates of ~80% at 80 bits, ~30% at 128 bits, and ~13% at 160 bits.
+Lastly, for results still "probably prime", an additional random-base
+Miller-Rabin test is performed.
 
 The result is that many numbers will return 2 (definitely prime),
 and the numbers that return 1 (probably prime) have gone through
@@ -335,10 +336,12 @@ Takes a positive number as input and returns back either 0 (composite),
 2 (definitely prime), or 1 (probably prime).  A great deal of effort is
 taken to return either 0 or 2 for all numbers.
 
-The current method first uses BPSW and a small number of Miller-Rabin
-tests with random bases to weed out composites and provide a
-deterministic answer for tiny numbers (under C<2^64>).  A quick BLS75
-C<n-1> test is attempted, followed by ECPP.
+The current method first uses BPSW to find composites and provide a
+deterministic answer for tiny numbers (under C<2^64>).  If no
+certificate is required, LLR and Proth tests can be run, and small
+numbers (under approximately C<2^82>) can be satisfied with a
+deterministic Miller-Rabin test.  If the result is still not determined,
+a quick  BLS75 C<n-1> test is attempted, followed by ECPP.
 
 The time required for primes of different input sizes on a circa-2009
 workstation averages about C<3ms> for 30-digits, C<5ms> for 40-digit,
@@ -380,20 +383,26 @@ verification.  Proof types used include:
 
 =head2 is_pseudoprime
 
-Takes a positive number C<n> and a base C<a> as input, and returns 1 if
-C<n> is a probable prime to base C<a>.  This is the simple Fermat primality
-test.  Removing primes, given base 2 this produces the sequence
-L<OEIS A001567|http://oeis.org/A001567>.
+Takes a positive number C<n> and one or more non-zero positive bases as input.
+Returns C<1> if the input is a probable prime to each base, C<0> if not.
+This is the simple Fermat primality test.
+Removing primes, given base 2 this produces the sequence L<OEIS A001567|http://oeis.org/A001567>.
+
+=head2 is_euler_pseudoprime
+
+Takes a positive number C<n> and one or more non-zero positive bases as input.
+Returns C<1> if the input is an Euler probable prime to each base, C<0> if not.
+This is the Euler test, sometimes called the Euler-Jacobi test.
+Removing primes, given base 2 this produces the sequence L<OEIS A047713|http://oeis.org/A047713>.
 
 =head2 is_strong_pseudoprime
 
   my $maybe_prime = is_strong_pseudoprime($n, 2);
   my $probably_prime = is_strong_pseudoprime($n, 2, 3, 5, 7, 11, 13, 17);
 
-Takes a positive number as input and one or more bases.  Returns 1 if
-the input is a prime or a strong pseudoprime to all of the bases, and
-0 if not.  The base must be a positive integer.  This is often called
-the Miller-Rabin test.
+Takes a positive number C<n> and one or more non-zero positive bases as input.
+Returns C<1> if the input is a strong probable prime to each base, C<0> if not.
+This is often called the Miller-Rabin test.
 
 If 0 is returned, then the number really is a composite.  If 1 is
 returned, then it is either a prime or a strong pseudoprime to all
@@ -515,17 +524,38 @@ pseudoprimes than the extra-strong Lucas test.  However this is still only
 counterexamples have been found with any of the Lucas tests described.
 
 
+=head2 is_euler_plumb_pseudoprime
+
+Takes a positive number C<n> as input and returns 1 if C<n> passes
+Colin Plumb's Euler Criterion primality test.  Pseudoprimes to this test
+are a subset of the base 2 Fermat and Euler tests, but a superset
+of the base 2 strong pseudoprime (Miller-Rabin) test.
+
+The main reason for this test is that is a bit more efficient
+than other probable prime tests.
+
+
 =head2 is_perrin_pseudoprime
 
 Takes a positive number C<n> as input and returns 1 if C<n> divides C<P(n)>
 where C<P(n)> is the Perrin number of C<n>.  The Perrin sequence is defined by
+C<P(n) = P(n-2) + P(n-3)> with C<P(0) = 3, P(1) = 0, P(2) = 2>.
 
-   C<P(0) = 3, P(1) = 0, P(2) = 2;  P(n) = P(n-2) + P(n-3)>
-
-This is not a commonly used test, as it runs 5 to 10 times slower than most
+This is not a commonly used test, as it runs slower than most
 of the other probable prime tests and offers little benefit, especially over
-combined tests like L</is_bpsw_prime> and
-L</is_frobenius_underwood_pseudoprime>.
+combined tests like L</is_bpsw_prime>,
+L</is_frobenius_underwood_pseudoprime>, and
+L</is_frobenius_khashin_pseudoprime>.
+
+An optional second argument C<r> indicates whether to run additional tests.
+With C<r=1>, C<P(-n) = -1 mod n> is also verified, creating the
+"minimal restricted" test.
+With C<r=2>, the full signature is also tested using the Adams and Shanks (1982)
+rules (without the quadratic form test).
+With C<r=3>, the full signature is tested using the Grantham (2000) test, which
+additionally does not allow pseudoprimes to be divisible by 2 or 23.
+The minimal restricted pseudoprime sequence is L<OEIS A018187|http://oeis.org/A018187>.
+
 
 =head2 is_frobenius_pseudoprime
 
@@ -540,14 +570,24 @@ case, the discriminant C<a^2-4b> must not be a perfect square.
 
 Takes a positive number as input, and returns 1 if the input passes the
 efficient Frobenius test of Paul Underwood.  This selects a parameter C<a>
-as the least positive integer such that C<(a^2-4|n)=-1>, then verifies that
-C<(2+2)^(n+1) = 2a + 5 mod (x^2-ax+1,n)>.  This combines a Fermat and Lucas
+as the least non-negative integer such that C<(a^2-4|n)=-1>, then verifies that
+C<(x+2)^(n+1) = 2a + 5 mod (x^2-ax+1,n)>.  This combines a Fermat and Lucas
 test at a computational cost of about 2.5x a strong pseudoprime test.  This
-makes it similar to, but faster than, a Frobenius test.
+makes it similar to, but faster than, a standard Frobenius test.
 
-There are no known pseudoprimes to this test.  This test also has no overlap
-with the BPSW test, making it a very effective method for adding additional
-certainty.
+This test is deterministic (no randomness is used).  There are no known
+pseudoprimes to this test.  This test also has no overlap with the BPSW
+test, making it a very effective method for adding additional certainty.
+
+=head2 is_frobenius_khashin_pseudoprime
+
+Takes a positive number as input, and returns 1 if the input passes the
+Frobenius test of Sergey Khashin.  This ensures C<n> is not a perfect square,
+selects the parameter C<c> as the smallest odd prime such that C<(c|n)=-1>,
+then verifies that C<(1+D)^n = (1-D) mod n> where C<D = sqrt(c) mod n>.
+
+This test is deterministic (no randomness is used).
+There are no known pseudoprimes to this test.
 
 =head2 is_bpsw_prime
 
@@ -567,15 +607,79 @@ Takes a positive number as input, and returns 1 if the input passes the
 Agrawal-Kayal-Saxena (AKS) primality test.  This is a deterministic
 unconditional primality test which runs in polynomial time for general input.
 
-In theory, AKS is extremely important.  In practice, it is essentially
-useless.  Estimated run time for a 150 digit input is about 9 years,
-making the case that while the algorithmic complexity I<growth> is
-polynomial, the constants are ludicrously high.  There are some ideas
-of Bernstein that can reduce this a little, but it would still take years
-for numbers that ECPP or APR-CL can prove in seconds.
+The particular method used is theorem 4.1 from Bernstein (2003).  This is
+substantially faster than the original AKS publication, the later version
+with improvements by Lenstra (sometimes called the V6 paper), or the later
+improvements of Voloch and Bornemann.  It is, by a large order, faster than
+any other known implementation as of early 2016.
+
+For theoretical analysis of the primality task, AKS is extremely important.
+In practice, it is essentially useless.  Estimated run time for a 150 digit
+input is over 2 days, making the case that while the algorithmic complexity
+I<growth> is polynomial, the constants are extremely high.  It will take
+years for for numbers that ECPP or APR-CL can prove in seconds.
 
 Typically you should use L</is_provable_prime> and let it decide the method.
 
+=head2 is_mersenne_prime
+
+  say "2^607-1 (M607) is a Mersenne prime" if is_mersenne_prime(607);
+
+Takes a positive number C<p> as input and returns 1 if C<2^p-1> is prime.
+After some pre-testing, the Lucas-Lehmer test is performed.
+This is a deterministic unconditional test that runs very fast compared
+to other primality methods for numbers of comparable size, and vastly
+faster than any known general-form primality proof methods.
+
+=head2 is_llr_prime
+
+Takes a positive number C<n> as input and returns one of: 0 (definitely
+composite), 2 (definitely prime), or -1 (test does not indicate anything).
+This implements the Lucas-Lehmer-Riesel test for
+fast deterministic primality testing on numbers of the form C<k * 2^n - 1>.
+If C<k = 1> then this is a Mersenne number and the Lucas-Lehmer test is used.
+If the number is not of this form, or if C<k E<lt>= 2^n>, then C<-1> will
+be returned as the test does not apply.
+Otherwise, the LLR
+test is performed.  While not as fast as the Lucas-Lehmer test for Mersenne
+numbers, it is almost as fast as a single strong pseudoprime test (i.e.
+Miller-Rabin test) while giving a certain answer.
+
+=head2 is_proth_prime
+
+Takes a positive number C<n> as input and returns one of: 0 (definitely
+composite), 2 (definitely prime), or -1 (test does not indicate anything).
+This applies Proth's theorem for fast Las Vegas
+primality testing on numbers of the form C<k * 2^n + 1>.
+If the number is not of this form, or if C<k E<lt>= 2^n>, then C<-1> will
+be returned as the test does not apply.
+Otherwise, a search is performed to find a quadratic nonresidue modulo C<n>.
+If none can be found after a brief search, C<-1> is returned as no conclusion
+can be reached.  Otherwise, Proth's theorem is checked which conclusively
+indicates primality.
+While not as fast as the Lucas-Lehmer test for Mersenne
+numbers, it is almost as fast as a single strong pseudoprime test (i.e.
+Miller-Rabin test) while giving a certain answer.
+
+=head2 is_miller_prime
+
+  say "$n is definitely prime" if is_miller_prime($n);
+  say "$n is definitely prime assuming the GRH" if is_miller_prime($n, 1);
+
+Takes a positive number as input, and returns 1 if the input passes the
+deterministic Miller test.  An optional second argument indicates whether
+the Generalized Riemann Hypothesis should be assumed, and defaults to 0.
+Setting the verbose flag to 2 or higher will show how many bases are used.
+The unconditional test is exponential time, while the conditional test
+(assuming the GRH) is polynomial time.
+
+This is a very slow method in practice, and generally should not be used.
+The asymptotic complexity of the GRH version is good in theory, matching
+ECPP, but in practice it is much slower.  The number of bases used by
+the unconditional test grows quite rapidly, impractically many past about
+160 bits, and overflows a 64-bit integer at 456 bits -- sizes that are
+trivial for the unconditional APR-CL and ECPP tests.
+
 =head2 is_nminus1_prime
 
   say "$n is definitely prime" if is_nminus1_prime($n);
@@ -588,8 +692,38 @@ inputs (under 40 digits) this is typically very easy, and some numbers will
 naturally lead to this being very fast.  As the input grows, this method
 slows down rapidly.
 
+This method is most appropriate for numbers of the form C<k+1> where C<k>
+can be easily factored.
 Typically you should use L</is_provable_prime> and let it decide the method.
 
+=head2 is_nplus1_prime
+
+Takes a positive number as input, and returns 1 if the input passes either
+theorem 17 or theorem 19 of the Brillhart-Lehmer-Selfridge primality test.
+This is a deterministic unconditional primality test which requires factoring
+C<n+1> to a linear factor less than the cube root of the input.  For small
+inputs (under 40 digits) this is typically very easy, and some numbers will
+naturally lead to this being very fast.  As the input grows, this method
+slows down rapidly.
+
+Disregarding factoring, this is slightly slower than the C<n-1> methods.
+It is most appropriate for numbers of the form C<k-1> where C<k> can be
+easily factored.
+
+=head2 is_bls75_prime
+
+Takes a positive number as input, and returns 1 if the input passes one of
+the tests from the Brillhart-Lehmer-Selfridge (1975) paper.  These use
+partial factoring of C<n-1> and C<n+1>.  Currently the implementation will
+use one of:
+
+  N-1   Corollary 1, Theorem 5, Theorem 7
+  N+1   Corollary 8, Theorem 17, Theorem 19
+  Comb  Theorem 20
+
+This is appropriate for cases where either C<n-1> or C<n+1> can be easily
+factored, or when both of them have many small factors.
+
 =head2 is_ecpp_prime
 
   say "$n is definitely prime" if is_ecpp_prime($n);
@@ -618,8 +752,93 @@ Typically you should use L</is_provable_prime> and let it decide the method.
 Returns all the primes between the lower and upper limits (inclusive), with
 a lower limit of C<2> if none is given.
 
-An array reference is returned (with large lists this is much faster and uses
-less memory than returning an array directly).
+An array reference is returned, matching the signature of the function
+of the same name in L<Math::Prime::Util>.
+
+Values above 64-bit are extra-strong BPSW probable primes.
+
+
+=head2 sieve_primes
+
+  my @primes = sieve_primes(2**100, 2**100 + 10000);
+  my @candidates = sieve_primes(2**1000, 2**1000 + 10000, 40000);
+
+Given two arguments C<low> and C<high>, this returns the primes in the
+interval (inclusive) as a list.  It operates similar to L<primes>, though
+must always have an lower and upper bound and returns a list.
+
+With three arguments C<low>, C<high>, and C<limit>, this does a partial
+sieve over the inclusive range and returns the list that pass the sieve.
+If C<limit> is less than C<2> then it is identical to the two-argument
+version, in that a primality test will be performed after sieving.
+Otherwise, sieving is performed up to C<limit>.
+
+The two-argument version is typically only used internally and adds little
+functionality.  The three-argument version is quite useful for applications
+that want to apply their own primality or other tests, and wish to have a
+list of values in the range with no small factors.  This is quite common
+for applications involving prime gaps.
+
+Also see L</sieve_range>.
+
+
+=head2 sieve_range
+
+  my @candidates = sieve_range(2**1000, 10000, 40000);
+
+Given a start value C<n>, and native unsigned integers C<width> and C<depth>,
+a sieve of maximum depth C<depth> is done for the C<width> consecutive
+numbers beginning with C<n>.  An array of offsets from the start is returned.
+
+The returned list contains those offsets in the range C<n> to C<n+width-1>
+where C<n + offset> has no prime factors less than C<depth>.
+
+This function is very similar to the three argument form of L</sieve_primes>.
+The differences are using C<(n,width)> instead of C<(low,high)>, and most
+importantly returning small offsets from the start value rather than the
+values themselves.  This can substantially reduce overhead for
+multi-thousand digit numbers.
+
+
+=head2 sieve_twin_primes
+
+  my @primes = sieve_twin_primes(2**1000, 2**1000 + 500000);
+
+Given two arguments C<low> and C<high>, this returns each lower twin prime
+in the interval (inclusive).  The result is a list, not a reference.
+
+This does a partial sieve of the range, removes any non-twin candidates,
+then checks that each pair are both BPSW probable primes.  This is
+substantially more efficient than sieving for all primes followed by
+removing those that are not twin primes.
+
+
+=head2 sieve_prime_cluster
+
+  # Find some prime septuplets
+  my @s = sieve_prime_cluster(2**100, 2**100+1e12, 2,6,8,12,18,20);
+
+Efficiently finds prime clusters between the first two arguments C<low>
+and C<high> (inclusive).  The remaining arguments describe the cluster.
+The cluster values must be even, less than 31 bits, and strictly increasing.
+Given a cluster set C<C>, the returned values are all primes in the
+range where C<p+c> is prime for all C<c> in the cluster set C<C>.
+
+The cluster is described as offsets from 0, with the implicit prime
+at 0.  Hence an empty list is asking for all primes (the cluster
+C<p+0>).  A list with the single value C<2> will find all twin primes
+(the cluster where C<p+0> and C<p+2> are prime).  The list C<2,6,8>
+will find prime quadruplets.  Note that there is no requirement that
+the list denote a constellation (a cluster with minimal distance) --
+the list C<42,92,606> is just fine.
+
+For long clusters, e.g. L<OEIS series A213601|http://oeis.org/A213601>
+prime 12-tuplets, this will be immensely more efficient than filtering
+out the cluster from a list of primes.  For that example, a range of
+C<10^13> takes less than a second to search -- thousands of times faster
+than filtering results from primes or twin primes.
+Shorter clusters are not quite this efficient, and the overhead for
+returning large arrays should not be ignored.
 
 
 =head2 next_prime
@@ -646,6 +865,49 @@ The function L</is_prob_prime> is used to determine when a prime is found,
 hence the result is a probable prime (using BPSW).
 
 
+=head2 surround_primes
+
+  ($dprev, $dnext) = surround_primes($n);
+
+Returns the distances to the previous and next primes of the input C<n>.
+This is slightly more efficient than calling both L</prev_prime> and
+L</next_prime>, and returning the native integer distances can be more
+efficient with large inputs.
+
+If an optional second argument C<d> is given, and the input C<n> is larger
+than C<2^64>, then if a SPSP-2 is found in the range C<n-d> to C<n+d>
+(inclusive) then it will be returned with the other argument set to C<0>.
+Otherwise, the first SPSP-2 values found are returned.
+This feature is especially useful for prime gap searches as well as
+finding the nearest prime to a value.
+
+Note that with a non-zero second argument, the values returned have not
+undergone a full BPSW test; just sieving and a SPSP-2 test.
+
+
+=head2 lucasu
+
+  say "Fibonacci($_) = ", lucasu(1,-1,$_) for 0..100;
+
+Given integers C<P>, C<Q>, and the non-negative integer C<k>,
+computes C<U_k> for the Lucas sequence defined by C<P>,C<Q>.  These include
+the Fibonacci numbers (C<1,-1>), the Pell numbers (C<2,-1>), the Jacobsthal
+numbers (C<1,-2>), the Mersenne numbers (C<3,2>), and more.
+
+This corresponds to OpenPFGW's C<lucasU> function and gmpy2's C<lucasu>
+function.
+
+=head2 lucasv
+
+  say "Lucas($_) = ", lucasv(1,-1,$_) for 0..100;
+
+Given integers C<P>, C<Q>, and the non-negative integer C<k>,
+computes C<V_k> for the Lucas sequence defined by C<P>,C<Q>.  These include
+the Lucas numbers (C<1,-1>).
+
+This corresponds to OpenPFGW's C<lucasV> function and gmpy2's C<lucasv>
+function.
+
 =head2 lucas_sequence
 
   my($U, $V, $Qk) = lucas_sequence($n, $P, $Q, $k)
@@ -711,6 +973,19 @@ the values satisfying Bézout's Identity.
 This corresponds to Pari's C<gcdext> function, which was renamed from
 C<bezout> out Pari 2.6.  The results will hence match L<Math::Pari/bezout>.
 
+=head2 chinese
+
+  say chinese( [14,643], [254,419], [87,733] );  # 87041638
+
+Solves a system of simultaneous congruences using the Chinese Remainder
+Theorem (with extension to non-coprime moduli).  A list of C<[a,n]> pairs
+are taken as input, each representing an equation C<x ≡ a mod n>.  If no
+solution exists, C<undef> is returned.  If a solution is returned, the
+modulus is equal to the lcm of all the given moduli (see L</lcm>.  In
+the standard case where all values of C<n> are coprime, this is just the
+product.  The C<n> values must be positive integers, while the C<a> values
+are integers.
+
 =head2 vecsum
 
 Returns the sum of all arguments, each of which must be an integer.
@@ -752,10 +1027,31 @@ C<n E<lt> 0, k > 0> are undefined.
 =head2 bernfrac
 
 Returns the Bernoulli number C<B_n> for an integer argument C<n>, as a
-rational number represented by two L<Math::BigInt> objects.  B_1 = 1/2.
+rational number.  Two values are returned, the numerator and denominator.
+B_1 = 1/2.
 This corresponds to Pari's C<bernfrac(n)> and Mathematica's C<BernoulliB>
 functions.
 
+=head2 bernreal
+
+Returns the Bernoulli number C<B_n> for an integer argument C<n>, as a
+string floating point.  An optional second argument indicates the number
+of significant digits to be used, with the result rounded.  The default
+is 40 digits.
+
+=head2 harmfrac
+
+Returns the Harmonic number C<H_n> for an integer argument C<n>, as a
+rational number.  Two values are returned, the numerator and denominator.
+numbers are the sum of reciprocals of the first C<n> natural numbers:
+C<1 + 1/2 + 1/3 + ... + 1/n>.
+
+=head2 harmreal
+
+Returns the Harmonic number C<H_n> for an integer argument C<n>, as a
+string floating point.  An optional second argument indicates the number
+of digits to be preserved past the decimal place, with a default of 40.
+
 =head2 stirling
 
   say "s(14,2) = ", stirling(14, 2);
@@ -774,6 +1070,25 @@ of the second kind are the number of ways to partition a set of C<n>
 elements into C<k> non-empty subsets.  The Lah numbers are the number of
 ways to split a set of C<n> elements into C<k> non-empty lists.
 
+=head2 zeta
+
+Given a positive integer or float C<n>, returns the real Riemann Zeta
+value as a string floating point.  An optional second argument indicates
+the number of digits past the decimal point (default 40).
+
+The implementation is algorithm 2 of Borwein (1991).  Performance with
+integer inputs is good, but floating point arguments with high precision
+will be much slower than methods using MPFR.  L<Math::Prime::Util> will
+try to use L<Math::MPFR> if possible.
+
+=head2 riemannr
+
+Given a positive integer or float C<n>, returns the real Riemann R function
+as a string floating point.  An optional second argument indicates the number
+of significant digits (default 40) with the result rounded.
+
+The implementation is the standard Gram series.
+
 
 =head2 znorder
 
@@ -800,6 +1115,37 @@ is a list of the smallest primitive roots, which is what this function
 produces.
 
 
+=head2 is_primitive_root
+
+Given two non-negative numbers C<a> and C<n>, returns C<1> if C<a> is a
+primitive root modulo C<n>, and C<0> if not.  If C<a> is a primitive root,
+then C<euler_phi(n)> is the smallest C<e> for which C<a^e = 1 mod n>.
+
+
+=head2 sigma
+
+  say "Sum of divisors of $n:", sigma( $n );
+  say "sigma_2($n) = ", sigma($n, 2);
+  say "Number of divisors: sigma_0($n) = ", sigma($n, 0);
+
+This function takes a positive integer as input and returns the sum of
+its divisors, including 1 and itself.  An optional second argument C<k>
+may be given, which will result in the sum of the C<k-th> powers of the
+divisors to be returned.
+
+This is known as the sigma function (see Hardy and Wright section 16.7,
+or OEIS A000203).  The API is identical to Pari/GP's C<sigma> function.
+This function is useful for calculating things like aliquot sums, abundant
+numbers, perfect numbers, etc.
+
+
+=head2 ramanujan_tau
+
+Takes a positive integer as input and returns the value of Ramanujan's tau
+function.  The result is a signed integer.
+This corresponds to Mathematica's C<RamanujanTau> function.
+
+
 =head2 valuation
 
   say "$n is divisible by 2 ", valuation($n,2), " times.";
@@ -835,6 +1181,47 @@ Given two integers C<a> and C<n>, return the inverse of C<a> modulo C<n>.
 If not defined, undef is returned.  If defined, then the return value
 multiplied by C<a> equals C<1> modulo C<n>.
 
+=head2 sqrtmod
+
+Given two integers C<a> and C<p>, return the square root of C<a> mod C<p>.
+If no square root exists, undef is returned.  If defined, the return value
+C<s> will always satisfy C<mulmod(s,s,p) = a>.
+
+If C<p> is not a prime, it is possible no result will be returned even
+though a modular root exists.
+
+Only one root is returned, even though there are at least two.  In the
+case of C<p> a prime and a return value C<s>, then both C<+s mod n> and
+C<-s mod n> are roots.  The least C<s> will be returned.  In the case of
+composites, many roots may exist, but only one will be returned.
+
+=head2 addmod
+
+Given three integers C<a>, C<b>, and C<n> where C<a> and C<n> are unsigned,
+return C<(a+b) mod n>.  This is particularly useful when dealing with
+numbers that are larger than a half-word but still native size.  No
+bigint package is needed and this can be 10-200x faster than using one.
+
+=head2 mulmod
+
+Given three integers C<a>, C<b>, and C<n> where C<a> and C<n> are unsigned,
+return C<(a*b) mod n>.  This is particularly useful when C<n> fits in a
+native integer.  No bigint package is needed and this can be 10-200x
+faster than using one.
+
+=head2 powmod
+
+Given three integers C<a>, C<b>, and C<n> where C<a> and C<n> are unsigned,
+return C<(a ** b) mod n>.  Typically binary exponentiation is used, so
+the process is very efficient.  With native size inputs, no bigint
+library is needed.
+
+=head2 divmod
+
+Given three integers C<a>, C<b>, and C<n> where C<a> and C<n> are unsigned,
+return C<(a/b) mod n>.  This is done as C<(a * (1/b mod n)) mod n>.  If
+no inverse of C<b> mod C<n> exists then undef if returned.
+
 
 =head2 consecutive_integer_lcm
 
@@ -947,6 +1334,25 @@ and 0 otherwise.  For example, if C<k=2> then this detects perfect squares.
 This corresponds to Pari/GP's C<ispower> function, with the limitations of
 only integer arguments and no third argument may be given to return the root.
 
+=head2 is_prime_power
+
+Given an integer input C<n>, returns C<k> if C<n = p^k> for some prime p,
+and zero otherwise.
+
+This corresponds to Pari/GP's C<isprimepower> function.
+
+=head2 sqrtint
+
+Returns the truncated integer part of the square root of C<n>.
+
+This corresponds to Pari/GP's C<sqrtint> function.
+
+=head2 rootint
+
+Given C<n> and C<k>, returns the truncated integer part of the C<k-th> root
+of C<n>.
+
+This corresponds to Pari/GP's C<sqrtnint> function.
 
 
 =head2 factor
@@ -979,6 +1385,23 @@ L<gmp-ecm|http://ecm.gforge.inria.fr/>, and
 L<GGNFS|http://sourceforge.net/projects/ggnfs/>.
 
 
+=head2 divisors
+
+  my @divisors = divisors(30);   # returns (1, 2, 3, 5, 6, 10, 15, 30)
+
+Produces all the divisors of a positive number input, including 1 and
+the input number.  The divisors are a power set of multiplications of
+the prime factors, returned as a sorted list with no duplications.
+The result is identical to that of Pari's C<divisors> and
+Mathematica's C<Divisors[n]> functions.
+
+In scalar context this returns the sigma0 function (OEIS A000005), and
+is the same result as evaluating the returned array in scalar context
+(but much more efficient).
+The result then corresponds to Pari's C<numdiv> and Mathematica's
+C<DivisorSigma[0,n]> functions.
+
+
 =head2 trial_factor
 
   my @factors = trial_factor($n);
@@ -1107,7 +1530,7 @@ This is Hart's One Line Factorization method, which is a variant of Fermat's
 algorithm.  A premultiplier of 480 is used.  It is very good at factoring
 numbers that are close to perfect squares, or small numbers.  Very naive
 methods of picking RSA parameters sometimes yield numbers in this form, so
-it can be useful to run this a few rounds to check.  For example, the number:
+it can be useful to run a few rounds to check.  For example, the number:
 
   18548676741817250104151622545580576823736636896432849057 \
   10984160646722888555430591384041316374473729421512365598 \
@@ -1184,6 +1607,22 @@ However, it is substantially faster than the other methods on large inputs
 having large factors, and is the method of choice for 35+ digit semiprimes.
 
 
+=head2 todigits
+
+Given an integer C<n>, return an array of digits of C<|n|>.  An optional
+second integer argument specifies a base (default 10).  For example,
+given a base of 2, this returns an array of binary digits of C<n>.
+An optional third argument specifies a length for the returned array.
+The result will be either have upper digits truncated or have leading
+zeros added.
+
+C<todigits(0)> returns an empty array.
+The base must be at least 2, and is limited to an int.
+Length must be at least zero and is limited to an int.
+
+
+
+
 =head1 SEE ALSO
 
 =over 4
@@ -1239,6 +1678,9 @@ There are now better alternatives.
 
 =item Robert Baillie and Samuel S. Wagstaff, Jr., "Lucas Pseudoprimes", Mathematics of Computation, v35 n152, October 1980, pp 1391-1417.  L<http://mpqs.free.fr/LucasPseudoprimes.pdf>
 
+=item Daniel J. Bernstein, "Proving Primality After Agrawal-Kayal-Saxena",
+preprint, Jan 2003.  L<http://cr.yp.to/papers/aks.pdf>
+
 =item Jon Grantham, "Frobenius Pseudoprimes", Mathematics of Computation, v70 n234, March 2000, pp 873-891.  L<http://www.ams.org/journals/mcom/2001-70-234/S0025-5718-00-01197-2/>
 
 =item John Brillhart, D. H. Lehmer, and J. L. Selfridge, "New Primality Criteria and Factorizations of 2^m +/- 1", Mathematics of Computation, v29, n130, Apr 1975, pp 620-647.  L<http://www.ams.org/journals/mcom/1975-29-130/S0025-5718-1975-0384673-1/S0025-5718-1975-0384673-1.pdf>
@@ -1298,7 +1740,7 @@ ECM implementation, as well as the papers by Brent and Montgomery.
 
 =head1 COPYRIGHT
 
-Copyright 2011-2014 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
+Copyright 2011-2016 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
